/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("dijit.form.DateTextBox"); // dojo.widget.DropdownDatePicker

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui2.internal.presentation.WorkItemLayout");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
// Section Layout required here for debugging
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionLayout");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TwoColumnLayout");

dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.EditorPresentation");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItem");
dojo.require("com.ibm.team.workitem.web.mvvm.model.WorkItemWorkingCopy");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");


(function() {
	
var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var EditorUtil				= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var LayoutUtil            	= com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ImagePool				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var isGuest 				= com.ibm.team.repository.web.client.session.isGuest;
var EditorPresentation		= com.ibm.team.workitem.web.ui.internal.EditorPresentation;
var WorkItem				= com.ibm.team.workitem.web.client.internal.WorkItem;
var WorkItemWorkingCopy = com.ibm.team.workitem.web.mvvm.model.WorkItemWorkingCopy;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemOverview.html"),
		/*
		 * The sectionMap is used to map the sections to the overview layout
		 * 
		 */
		sectionMap: {
				details: {
						column: "left",
					 	renderFunctionName: "_renderDetails",
						exclude: {
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.AttachmentsPart" : true,
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart" : true,
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart" : false,
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart" : false
						}
						},
				linksSummary: { // Quick Information
						highlightBackground: true,
						column: "right",
						className: "QuickInformationSection",
					 	renderFunctionName: "_renderGenericSection"
						},
				quickInfo: {  // Quick Information
						highlightBackground: true,
						column: "right",
						className: "QuickInformationSection",
					 	renderFunctionName: "_renderGenericSection"
						},
				defaultSection: {
						column: "left",
					 	renderFunctionName: "_renderDescription"
						},
				description: {
						column: "left",
					 	renderFunctionName: "_renderDescription"
						},
				discussion: {
						column: "left",
					 	renderFunctionName: "_renderDiscussion"
						}
			},
		/*
		 * The modifierMap is used to build the modifiers object passed to the widget
		 * It includes formatting and control information not defined in the 
		 * process spec.  
		 * spanColumns: True if the widget needs to span both columns
		 * 
		 * noValueLabel: Label to show when a label or date has no value
		 * noValueTitle: Title/tooltip to show when a label or date has no value
		 */		
		_modifierMap: {
				description: {
//					noValueLabel: "No Description."  See constructor
					},
				creationDate: { // Created
//					noValueLabel: "Now",   // See Constructor
//					noValueTitle: "This work item has not yet been saved." // See constructor
					},
				resolver: {  // Resolved BY
//					noValueLabel: "Not Resolved"  // See Constructor
					},
				resolutionDate: {
//					noValueLabel: // See Constructor
					},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart: {
//					rowWeight: 2,  // counts as multiple rows
					skipNextSeparator: true,
//					attributeLabel: "Project Area",  // see constructor
					isProjectAreaWidget: true,
					bidiSupport: true,
					addAttributesAfter: [ {
						isTeamAreaWidget: true,
						isProjectAreaWidget: false,
						attributeId: "com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart_TeamArea"
//						attributeLabel: "Team Area", // see constructor
//						valuesFunction: "_getTeamAreaValues"
						} ]
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_HTMLAttributePart: {
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_RichTextAttributePart: {
				},
 				com_ibm_team_workitem_web_ui_internal_view_layout_SimpleRichTextWidget: {
 					isSingleColumnWide: true
 				},
 				com_ibm_team_workitem_web_ui_internal_view_editor_parts_LinksPart: {
 					isSingleColumnWide: true,
					isSpanningColumns: true
 				},
 				com_ibm_team_workitem_web_ui_internal_view_editor_parts_DiscussionPart: {
 					isSingleColumnWide: true,
					isSpanningColumns: true
 				},
 				com_ibm_team_workitem_web_ui2_internal_presentation_Wiki: {
 					isSingleColumnWide: true,
					isSpanningColumns: true 					
 				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_QuickInformationPart: {
					isSingleColumnWide: true,
					isSpanningColumns: true
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_SeparatorPart: {
					rowClass: "SeparatorTableRow",
					skipLabel: true,
					skipRequiredMarker: true
				},
				com_ibm_team_workitem_web_ui_internal_view_layout_LineSeparatorWidget: {
					rowClass: "SeparatorTableRow",
					skipLabel: true,
					skipRequiredMarker: true,
					isSpanningColumns: true,
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui_internal_view_layout_StaticTextWidget: {
					rowClass: "SeparatorTableRow",
					skipLabel: true,
					skipRequiredMarker: true,
					isSpanningColumns: true,
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui_internal_view_layout_StaticRichTextWidget: {
					rowClass: "SeparatorTableRow",
					skipLabel: true,
					skipRequiredMarker: true,
					isSpanningColumns: true,
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui2_internal_presentation_ReferencedWorkItemList: {
					skipLabel: false,
					skipRequiredMarker: true,
					isSpanningColumns: false,
					isSingleColumnWide: true
				},
				com_ibm_team_tpt_web_ui_timesheet_internal_TimeSheetEntryPartWeb: {
					skipLabel: true,
					skipRequiredMarker: true,
					isSpanningColumns: true,
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_ApprovalsPart: {
					skipLabel:  false,
					skipRequiredMarker: false,
					isSpanningColumns: true,
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_WorkItemListPart: {
					skipLabel: false,
					skipRequiredMarker: false,
					isSpanningColumns: false,
					isSingleColumnWide: true
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_HistoryPart: {
					skipLabel:  false,
					skipRequiredMarker: true,
					isSpanningColumns: true,
					isSingleColumnWide: true
				}
			},
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.buttonSaveText = this.messages.buttonSave;
			this._modifierMap.description.noValueLabel = this.messages.labelNoDescription;
			this._modifierMap.creationDate.noValueLabel = this.messages.labelNow;
			this._modifierMap.creationDate.noValueTitle = this.messages.titleNotSavedYet;
			this._modifierMap.resolutionDate.noValueLabel = this.messages.labelNotResolved;
			this._modifierMap.resolver.noValueLabel = this.messages.labelNotResolved;
			this._modifierMap.com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart.attributeLabel = this.messages.labelProjectAreaAttribute;
			this._modifierMap.com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart.addAttributesAfter[0].attributeLabel = this.messages.labelTeamAreaAttribute;
		},
		sortSections: function(sections){
			var bucketedSections= {"details": [], "linkSummary": [], "quickInfo": [], "defaultSection": [], "description": [], "discussion": [], "__other_slots": []};
			for(var i=0; i< sections.length; i++){
				if (bucketedSections[sections[i].slot]) {
					bucketedSections[sections[i].slot].push(sections[i]);
				} else {
					bucketedSections["__other_slots"].push(sections[i]);
				}
			}
			var sortedSections= [];
			for(bucket in bucketedSections){
				sortedSections= sortedSections.concat(bucketedSections[bucket]);
			}
			return sortedSections;
		},
		postCreate: function() {
			var _columnLayout= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.layout.TwoColumnLayout({});
			this.allWidgets = {_columnLayout: _columnLayout };
			this._workItem = this.workItem;
			this.pageProps.sections= this.sortSections(this.pageProps.sections);
			if(this.pageProps && this.pageProps.sections) {
				for(var i=0;i<this.pageProps.sections.length;i++) {
					LayoutUtil.layoutSection(this, this.pageProps.sections[i], this.sectionMap, _columnLayout, this.parentController, this.allWidgets);
				}				
			}
			if(this.retainedState) {
				this.notifyChildren("setFormElementValues",this.retainedState, isGuest());
			} else {
				this.retainedState = {};
			}

			this._columnDIV.appendChild(_columnLayout.domNode);
		},
		startup: function() {
			for (var widget in this.allWidgets) {
				if (this.allWidgets[widget].startup) {
					this.allWidgets[widget].startup();
				}
			}
		},
		destroyWidget: function()  {
			for(var widget in this.allWidgets) {
				if(this.allWidgets[widget].destroyWidget) {
					this.allWidgets[widget].destroyWidget();					
				} else {
				if (this.allWidgets[widget].destroy) {
					this.allWidgets[widget].destroy();
				}
				}
			}
			this.destroy();			
		},
	// Called when the tab is shown
		showWidget: function() {
			if(this.isShown) return;
			this.isShown = true; // only do this once
			this.notifyChildren("showWidget");
		}, 

	// TODO: Remove this function
	_getTeamAreaValues: function(values, possibleValues, modifiers) {
		var value;
		try {
			value = this._workItem.attributes.teamArea.label;
		} 
		catch (e) {
		} // ignore
		return { values: [value] }
	},
    _addWidgetRow: function(presentation, displaySpec, tableNode,  values, possibleValues, isRequired, sectionLayout ) {
			var creationSpec = LayoutUtil.getPresentationWidget(presentation);

			
			var editorPresentation= new EditorPresentation(presentation);
			var workItem= new WorkItem(this.parentController.workingCopy);
			var hideIfEmpty= editorPresentation.getBooleanProperty("hideIfEmpty");
			var labelVisible= editorPresentation.getBooleanPropertyWithDefault("labelVisible", true);
			var doHide= !editorPresentation.isVisible(workItem._proxy.getWorkingCopy());
		
// template:
//			<tr>
//				<td class="AttributeTextLeft">Type: <span dojoAttachPoint="typeRequiredMarker" class="RequiredMarker"></span></td>
//				<td>
//					<select dojoAttachPoint="type" dojoAttachEvent="onchange: _handleSelectChangeWithImage" class="OverviewFormElement"></select>
//				</td>
//			</tr>
			
			var row = tableNode.insertRow(-1);
			if(doHide) {
				row.style.display = "none";
			}
			if(displaySpec["rowClass"]) {
				row.className=displaySpec["rowClass"];
			} else {
				row.className="LabelValueTableRow";
			}
			
			var labelColumn = row.insertCell(-1);
			labelColumn.className="Column LabelColumn";
						
			var labelSpan = document.createElement("span");

			var modifiers = {};
	
			if(displaySpec.isTeamAreaWidget) {
				modifiers.isTeamAreaWidget = true;
			}
			if(displaySpec.isProjectAreaWidget) {
				modifiers.isProjectAreaWidget = true;
			}			
	
			displaySpec.labelSpan = labelSpan;
			
			// presentation spec overrides displaySpec.
			// displaySpec will have things like the widget-defined label to use if the presentation lacks label
			// Handle hard coded labels
			if(!presentation.attributeLabel && displaySpec.attributeLabel) {
				presentation.attributeLabel = displaySpec.attributeLabel;
			}
			
			if(!displaySpec.skipLabel && labelVisible) {
				if(presentation.attributeLabel) {
					if(presentation.attributeLabel.length > 0) {
							var text = document.createTextNode(dojo.string.substitute(this.messages.labelValue, [ presentation.attributeLabel ]));
							if(presentation.attributeId){
									//Set up the attributeLabel to be the HTML label for the form element.
									var labelElement = document.createElement("LABEL");
									labelElement.appendChild(text);
									labelSpan.appendChild(labelElement);
							} else {
								labelSpan.appendChild(text);
							}
						} else {
							labelSpan.innerHTML = "";  /* innerHTML OK */
						}
				} else {
					displaySpec.hasDeferredLabel = true;
					labelSpan.innerHTML = "";  /* innerHTML OK */
				}
				if(presentation.presentationDescription != null){
					labelSpan.title = presentation.presentationDescription;	
				}
				labelColumn.appendChild(labelSpan);
			}
					
			if (!displaySpec.skipRequiredMarker && presentation.attributeId) {
				var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
				{workingCopy: this.parentController.workingCopy,
			 		attributeId: presentation.attributeId}
				);
				this.allWidgets[reqMark.id] = reqMark; // to be destroyed
				labelColumn.appendChild(reqMark.domNode);
			}

			if (creationSpec.modifiers) {
				dojo.mixin(modifiers, creationSpec.modifiers);
			}
			
			// If the label is not visible, make this wide
			if(modifiers.isSpanningColumns || displaySpec.isSpanningColumns || !labelVisible) {
				if(!displaySpec.isSpanningColumns) labelColumn.appendChild(document.createElement("br"));
				labelColumn.className="Column LabelColumn2";
				labelColumn.colSpan = "2";
				valueColumn = labelColumn;
			} else {
				var valueColumn = row.insertCell(-1);
				valueColumn.className="Column ValueColumn2";
			}
						
			if(presentation.widget) {
				var valueDiv = document.createElement("div");
				valueColumn.appendChild(valueDiv);
				this._addWidget(creationSpec, presentation, valueDiv, values, possibleValues, modifiers, sectionLayout, hideIfEmpty);
			} else {
				var valueSPAN = document.createElement("SPAN");
				valueSPAN.innerHTML ="&nbsp;";  /* innerHTML OK */
				valueColumn.appendChild(valueSPAN);
			}
		},
	    _addWidget: function(creationSpec, presentation, valueNode,  values, possibleValues, modifiers, sectionLayout, hideIfEmpty) {
			if (!creationSpec || !creationSpec.widget) {
				creationSpec= LayoutUtil.mergeObjects(creationSpec, LayoutUtil.getPresentationWidget(presentation));
			}

			var widgetClass= creationSpec.widget;

			var propertyModifiers= LayoutUtil.getPropertyMap(presentation, this._modifierMap[presentation.attributeId]);
			LayoutUtil.mergeObjects(modifiers, propertyModifiers);
			
			creationSpec.hideIfEmpty= hideIfEmpty;
			var self= this;
			if (widgetClass) {
    			var attributeId= propertyModifiers.attributeId != null ? propertyModifiers.attributeId : presentation.attributeId;
				attributeId= attributeId != null ? attributeId : presentation.widget.replace(/\./g, "_");
				if ((this._modifierMap[attributeId] && this._modifierMap[attributeId].bidiSupport)
						|| (!!presentation.attributeType && EditorUtil.getPreferableDirection(
								attributeId, presentation.attributeType) != "none")) {
					presentation.bidiSupport= true;
				}

				var currentAttribute= this.parentController.workingCopy.getAttribute(presentation.attributeId);
				var creationArgs= {
					parentWidget: this,
					creationSpec: creationSpec,
					workItem: this._workItem,
					workingCopy: this.parentController.workingCopy,
					parentController: this.parentController,
					attributeId: presentation.attributeId,
					attributeTypeId: currentAttribute ? currentAttribute.getAttributeTypeId() : null,
					attributeValue: this.parentController.workingCopy.getValue({
						path: [ "attributes", attributeId ]
					}),
					presentation: presentation,
					values: values,
					valueNode: valueNode,
					sectionLayout: sectionLayout,
					_modifiers: modifiers, // @adopted, See
											// item 170094
					_possibleValues: possibleValues
				}

				var managerSelector= this.parentController.getManagerSelector(presentation.attributeId);
				if (managerSelector) {
					creationArgs[managerSelector]= true;
				}

				if (creationSpec.isMissingWidget) {
					creationArgs.widgetName= creationSpec.widgetName;
					creationArgs.isMissingWidget= creationSpec.isMissingWidget;
				}

				try {
					var widget= this.parentController.createWidget(creationArgs);
				} catch (e) {
					var className= widgetClass;
					if (widgetClass.prototype && widgetClass.prototype.declaredClass) {
						className= widgetClass.prototype.declaredClass;
					}
					console.debug("*** ERROR creating '" + className + "': " + e.message);
					return;
				}

				if (presentation && presentation.displaySpec) {
					if (modifiers && modifiers.readonly == "true" && widget.setReadOnly) {
						widget.setReadOnly(true);
					}
					var displaySpec= presentation.displaySpec;

					if (widget.getLabel && displaySpec.labelSpan) {
						dojox.xml.parser.removeChildren(displaySpec.labelSpan);
						var text= document.createTextNode(widget.getLabel());
						displaySpec.labelSpan.appendChild(text);
					}

					if (displaySpec.hasDeferredLabel && displaySpec.labelSpan) {
						if (displaySpec.labelSpan.innerHTML == "" && presentation.attributeId) { /* innerHTML OK */
							var text= document.createTextNode(presentation.attributeId + ": ");
							displaySpec.labelSpan.appendChild(text);
						}
					}
				}
				if (sectionLayout) {
					sectionLayout.addChildWidget(widget);
				}

				var awKey= attributeId;
				if (displaySpec && displaySpec.attributeId) {
					awKey= displaySpec.attributeId;
				}
				var i= 0;
				var id= awKey; // Handle duplicate attributes
				while (this.allWidgets[id]) {
					id= awKey + i++;
				}
				this.allWidgets[id]= widget;
			}
		},
		// TODO: Remove this special case
		getDescriptionValue: function() {
			if(!this.allWidgets || !this.allWidgets.description ) return "";
			return this.allWidgets.description.getValue().getContent();
		},
		// TODO: Remove this special case
		_renderDescription: function(sectionNode, section, sectionLayout) {
			if(!section || !section.presentations || section.presentations.length ===0) return;

			//  Description section is a special case
			// if the attribute is description and the attribute label is the same as the section title
			// then hide the first label
			var propertyMap = LayoutUtil.getPropertyMap(section.presentations[0]);
			var hideFirstLabel = propertyMap.labelVisible=="false" ||  section.presentations.length === 1; 
			var sectionTitle;
			if (section.presentations && section.presentations.length > 0  &&  section.presentations[0].attributeId == "description" && section.presentations[0].attributeLabel ) {
				sectionTitle = this.messages.labelDescription;
				if (section.properties) {
					for (var k = 0; k < section.properties.length; k++) {
						if (section.properties[k].key == "title") {
							sectionTitle = section.properties[k].value;
							break;
						}
					}
				}
			}
			hideFirstLabel = hideFirstLabel || section.presentations.length ==  1 || section.presentations[0].attributeLabel==sectionTitle ;
			for(var i=0;i<section.presentations.length;i++) {
					var presentation = section.presentations[i];
					var hideLabel = presentation.widget == "com.ibm.team.workitem.web.ui.internal.view.layout.LineSeparatorWidget" ||
					                presentation.widget == "com.ibm.team.workitem.web.ui.internal.view.layout.StaticTextWidget" || 
									presentation.widget == "com.ibm.team.workitem.web.ui.internal.view.layout.StaticRichTextWidget";
					
					var attributeId = section.presentations[i].attributeId;
					var values = [];
					if(this.workItem && this.workItem.attributes && this.workItem.attributes[attributeId] ) {
						var attr = this.workItem.attributes[attributeId];
						if(dojo.isString(attr) || !dojo.isArray(attr)) {
							values = [ attr ];
						}
					}

					var hideIfEmpty = false;
					if(presentation && presentation.properties) {
						for(var k=0;k<presentation.properties.length;k++) {
							if(presentation.properties[k].key == "hideIfEmpty") {
								hideIfEmpty = presentation.properties[k].value == "true";
								break;
							}
						} 
					}	
					if(hideIfEmpty && (!values || values.length == 0 || (values[0].label!=null && this.parentController._isAttributeEqual(values[0].label, null))
					   	|| (values[0].userId && this.parentController._isAttributeEqual(values[0].userId, null)))){
						continue;
					}
					propertyMap = LayoutUtil.getPropertyMap(section.presentations[i]);
					if(hideLabel) propertyMap.labelVisible = "false";
					if(propertyMap.labelVisible !="false" && (i > 0 || (i==0 && !hideFirstLabel)) ) {
						if(i>0) {
							var hr = document.createElement("hr");
							hr.className = "subDescriptionRule";
							sectionNode.appendChild(hr);							
						}
						var titleNode = document.createElement("div");
						titleNode.className = "subDescriptionTitle";
						
						if(section.presentations[i].attributeLabel) {
							var title = section.presentations[i].attributeLabel;
						} else {
							var title = dojo.string.substitute(this.messages.labelDescriptionPlus, [ "" + i]);
						}
						titleNode.appendChild(document.createTextNode(title));

						var displaySpec = {};
						if(section.presentations[i] && section.presentations[i].attributeId) {
							var key = section.presentations[i].attributeId.replace(/\./g, "_");		
							if(this._modifierMap[key]) {
								displaySpec = this._modifierMap[key];			
							}
							if (!displaySpec.skipRequiredMarker) {
								var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
											{workingCopy: this.parentController.workingCopy,
									 		attributeId: section.presentations[i].attributeId}
										);
								this.allWidgets[reqMark.id] = reqMark; // to be distroyed
								titleNode.appendChild(reqMark.domNode);
							}
						}
						
						sectionNode.appendChild(titleNode);
					}
					var descriptionNode = document.createElement("div");
					sectionNode.appendChild(descriptionNode);

					/* If the label is hidden, put the required mark in the section header */
					if(hideFirstLabel) {
						sectionLayout.setRequiredAttributeId(attributeId, this.parentController.workingCopy);
					}
 
					if(attributeId == "description") {
						this.isDescriptionManager = true;
					}
					var hasSectionMenu = i==0 && hideFirstLabel && (
						section.presentations[0].widget.indexOf("HTMLAttributePart") != -1 ||
						section.presentations[0].widget.indexOf("RichTextAttributePart") != -1 ||
						section.presentations[0].widget.indexOf("com.ibm.team.workitem.web.ui2.internal.presentation.Wiki") != -1 || 
						section.presentations[0].widget.indexOf("StringAttributePart") != -1) ;
					
					var possibleValues = this._getPresentationPossibleValues(presentation);

					section.presentations[i].bidiSupport = true;
					this._addWidget({ setSectionMenu: hasSectionMenu, isDescription: true }, section.presentations[i], descriptionNode, values, possibleValues, {}, sectionLayout, hideIfEmpty);
			}
		},
		_getPresentationPossibleValues: function (pres) {
			var uiItems = this.parentController.workingCopy.getEditPropsValuesForAttribute(pres.attributeId, false);
			return uiItems || [];
		},
		_renderDiscussion: function(sectionNode, section, sectionLayout) {
			if(!section || !section.presentations || section.presentations.length ===0) return;

		    // TODO: Investigate if we can always use renderDescription, for now preserve current behavior
			if(section && section.presentations && (section.presentations.length > 1 || section.presentations[0].attributeId != "internalComments")) {
				return this._renderDescription(sectionNode, section, sectionLayout);
			}
			var discussionNode = document.createElement("div");
			sectionNode.appendChild(discussionNode);
			
			if(sectionLayout && sectionLayout.setRequiredAttributeId && section 
				&& section.presentations && section.presentations[0] && section.presentations[0].attributeId) {
				sectionLayout.setRequiredAttributeId(section.presentations[0].attributeId, this.parentController.workingCopy);
			}
			
			var values = this.workItem.attributes.internalComments && this.workItem.attributes.internalComments.items ? this.workItem.attributes.internalComments.items : [];
			
			section.presentations[0].bidiSupport = true;
			this._addWidget({iscommentManager: true}, section.presentations[0], discussionNode, values, [], {}, sectionLayout);
		},
		/*
		 * Show a value-less section (e.g., quick information)
		 */
		_renderGenericSection: function(sectionNode, section, sectionLayout) {
			if(!section || !section.presentations || section.presentations.length ===0) return;

			var contentNode = document.createElement("DIV");
			sectionNode.appendChild(contentNode);

			this._addWidget(null, section.presentations[0], contentNode, [], [], {}, sectionLayout);		
		},

		_renderDetails:  function(sectionNode, section, sectionLayout) {
			if(!section || !section.presentations) return;
			var firstLeft = true;
			var firstRight =true;
			var detailsSplitTableWrapper = document.createElement("div");
			detailsSplitTableWrapper.className = "detailsSplitTableWrapper";
			
			var detailsSplitTable = document.createElement("table");
			detailsSplitTable.className="DetailsSplitTable";
			var splitRow = detailsSplitTable.insertRow(-1);
			detailsSplitTableWrapper.appendChild(detailsSplitTable);
			
			var leftColumn = splitRow.insertCell(-1);
			var leftWidgetWrapper = document.createElement("div");
			leftWidgetWrapper.className = "DetailsInnerTableWrapper";
			var leftWidgetTable = document.createElement("table");
			leftWidgetTable.className = "DetailsInnerTable";
			leftWidgetWrapper.appendChild(leftWidgetTable);
			leftColumn.appendChild(leftWidgetWrapper);
			leftColumn.className = "Column leftColumn";

			var rightColumn = splitRow.insertCell(-1);
			var rightWidgetWrapper = document.createElement("div");
			rightWidgetWrapper.className = "DetailsInnerTableWrapper";
			var rightWidgetTable = document.createElement("table");
			rightWidgetTable.className = "DetailsInnerTable";
			rightWidgetWrapper.appendChild(rightWidgetTable);
			rightColumn.appendChild(rightWidgetWrapper);
			rightColumn.className = "Column rightColumn";
						
			var spanRow = detailsSplitTable.insertRow(-1);
			var spanColumn = spanRow.insertCell(-1);
			spanColumn.colSpan=2;
			spanColumn.className = "Column";
			var spanWidgetTable = document.createElement("table");
			spanWidgetTable.className="SpanWidgetTable";
			spanColumn.appendChild(spanWidgetTable);
			
			// First figure out how many columns in the left and right
			var weightedLength = 0;
			var skippedColumnCount = 0;
	
			for (var i = 0; i < section.presentations.length; i++) {
				var presentation = section.presentations[i];
				
				if (!presentation.widget) {
					skippedColumnCount++;
					continue;
				}
				
				var displaySpec = {};
				if(presentation.attributeId) {
					var key = presentation.attributeId.replace(/\./g, "_");		
					if(this._modifierMap[key]) {
						displaySpec = this._modifierMap[key];			
					}
				}
				presentation.displaySpec = displaySpec;			
	
				if(displaySpec.isInAlternateTable) {
					skippedColumnCount++;
				} else {
					if(displaySpec.rowWeight) {
						weightedLength = weightedLength + displaySpec.rowWeight; 
					} else {
						weightedLength++;
					}
				}
			}

			var isSingleColumn = false;
			var rightColumnCount = Math.floor(weightedLength / 2) ;
			rightColumnCount = this._getCorrectedRightColumnCount(section, rightColumnCount);
			if(rightColumnCount == 0) {
				rightColumn.width ="1px;"
				isSingleColumn = true;
			}
			
			var leftColumnCount = weightedLength - rightColumnCount;
			leftColumnCount = this._getCorrectedLeftColumnCount(section, leftColumnCount);
			if(leftColumnCount == 0) {
				leftColumn.width ="1px;"
				isSingleColumn = true;
			}
			
			
			var isIE6 = dojo.isIE && dojo.isIE < 7;
			
			weightedLength = 0;
			for(var i=0; i< section.presentations.length; i++) {
					var presentation = section.presentations[i];
					if(!presentation.widget) {
						skippedColumnCount++;
						continue; 
					}

					var displaySpec = presentation.displaySpec;

					var values = [];
					if(presentation.attributeId) {
						// priority and severity get mapped to internalSeverity and internalPrioirty
	    				var attributeId = displaySpec.attributeId != null ? displaySpec.attributeId : presentation.attributeId;
						values = this.workItem.attributes[attributeId] == null ? [] : this.workItem.attributes[attributeId];
						if(!dojo.isArray(values)) {
							values = [ values ];
						}
					}
					
					var possibleValues = this._getPresentationPossibleValues(presentation);
					
					if(presentation.widget) {
						var key = presentation.widget.replace(/\./g, "_");
						if(this._modifierMap[key]) {
							displaySpec = LayoutUtil.mergeObjects(displaySpec, this._modifierMap[key]);
						}				
					}

// Fill the left column first then fill the right column.
// Some values can be skipped and some may fall into the span section (tags)
//					spanWidgetTable, leftWidgetTable, rightWidgetTable

					if(displaySpec.rowWeight) {
						weightedLength = weightedLength + displaySpec.rowWeight;
					} else {
						weightedLength++;
					}
					
					var skipWidget = false;
					var isSep = key == "com_ibm_team_workitem_web_ui_internal_view_editor_parts_SeparatorPart";
					if(isSep && this._skipNextSeparator) {
						this._skipNextSeparator = false;
						continue;
					}

					try {
						if (this.sectionMap[section.slot] && this.sectionMap[section.slot].exclude[presentation.widget]) {
							console.debug(dojo.string.substitute(this.messages.errorPartNotSupportedInSlot, [presentation.widget, section.slot]));
							skipWidget = true;
						}
					} 
					catch (e) {	} // ignore 

					if (displaySpec.skipNextSeparator) {
						this._skipNextSeparator = true;
					} else {
						this._skipNextSeparator = false;						
					}
					
					var widgetTableWrapper;
					var widgetTableNode;
					if(displaySpec.isInAlternateTable) { 
						widgetTableNode = spanWidgetTable;
					} else {
						if( weightedLength <= leftColumnCount) {
							if (!(firstLeft && isSep)) {
								widgetTableWrapper = leftWidgetWrapper;
								widgetTableNode = leftWidgetTable;
								firstLeft = false;
							} else {
								skipWidget = true;
							}
						} else  {
							if (!(firstRight && isSep)) {
								widgetTableWrapper = rightWidgetWrapper;
								widgetTableNode = rightWidgetTable;
								firstRight = false;
							} else {
								skipWidget = true;
							}
						}
					}
					if(!skipWidget) {
						/*
						 * Team area is a special case where the label comes in wrong from the presentation
						 */
						if( presentation.partId == 'com.ibm.team.workitem.presentation.teamArea') {
							presentation.attributeLabel = displaySpec.attributeLabel;
						}
						sectionLayout._isDetails = true; // Do not let quick info set the background
						this._addWidgetRow(presentation, displaySpec, widgetTableNode, values, possibleValues, presentation.isRequired, sectionLayout );
						/*
						 * If it is in an H-Layout and a wide field, remove the width
						 */
						if(this.isHLayout && widgetTableWrapper && displaySpec.isSingleColumnWide) {
							dojo.removeClass(widgetTableWrapper, "DetailsInnerTableWrapper");
							dojo.addClass(widgetTableWrapper, "DetailsInnerTableWrapperWide");
						}
						/*
						 * Some presentations require two lines (progress)
						 */
						if(displaySpec.addAttributesAfter) {
							for(var j=0;j<displaySpec.addAttributesAfter.length;j++) {
								var presentation2 = dojo.mixin(presentation, {});
								presentation2.attributeLabel = displaySpec.addAttributesAfter[j].attributeLabel;
								var displaySpec2 = dojo.mixin(displaySpec, {});
								var displaySpec2 = dojo.mixin(displaySpec2, displaySpec.addAttributesAfter[j]);

								var result = {};
								if(displaySpec.addAttributesAfter[j].valuesFunction) {
									var result = this[displaySpec.addAttributesAfter[j].valuesFunction](values, possibleValues, displaySpec2) || {};
								}
								var values2 = result.values || values;
								var possibleValues2 = result.possibleValues || possibleValues;
								var displaySpec2 = result.modifiers || displaySpec2;
								presentation2.displaySpec = displaySpec2;
								this._addWidgetRow(presentation2, displaySpec2, widgetTableNode, values2, possibleValues2, presentation2.isRequired, sectionLayout );
							}
						}
						
					}
			}

			if(spanWidgetTable.rows && leftWidgetTable.rows) {
				var splitRow0=spanWidgetTable.rows[0];
				var leftRow0=leftWidgetTable.rows[0];
				if (splitRow0 && leftRow0) {
					if (splitRow0.childNodes[0].width < leftRow0.childNodes[0].width) {
						splitRow0.childNodes[0].width = leftRow0.childNodes[0].width;
					}
					else {
						leftRow0.childNodes[0].width = splitRow0.childNodes[0].width;
					}
				}				
			}
			sectionNode.appendChild(detailsSplitTableWrapper);
		},
		/*
		 * Allow subclasses to correct the details column counts
		 */
		_getCorrectedLeftColumnCount: function(section, leftColumnCount) {
			return leftColumnCount;
		},
		_getCorrectedRightColumnCount: function(section, rightColumnCount) {
			return rightColumnCount;
		},
		
		findObjectWithProperty: function(/* property, path */ parms) {
			var object; 
			if (this[parms.property]) {
				object = this;
			} else {
				object = EditorUtil.findObjectWithPropertyInObject({
				object: this.allWidgets,
				path: parms.path,
				property: parms.property
					}, this); 
			}
			return object; 
		},
		disableBottomSaveCommand: function(isDisabled) {
			this._saveCmdBottom.disabled= isDisabled = isDisabled;
			this._saveCmdBottom.style.display= "block";
		},
		// return an object containing form name-value pairs
		getFormElements: function() {		// return an object containing form name-value pairs
			var formElementValues= {};
			var self = this;
			this.notifyChildren("getFormElementValues",formElementValues );

			return formElementValues;
		},
		// TODO: Remove this special case
		_getMailto: function(address) {
			if(!address || address.length == 0) return null;
			return this.parentController._getMailto(address);
		}, 
		getRetainedState: function(retainedState) {
			this.notifyChildren("getRetainedState", retainedState);
		},
		hasRetainedState: function(retainedState) {
			return this.pollChildren("hasRetainedState", retainedState); 
		}, 
		hasUnsavedChanges: function() {
			return this.pollChildren("hasUnsavedChanges");
		},
		markFullyLoaded: function() {
			this.notifyChildren("markFullyLoaded");
		},
		notifyChildren: function(functionName, parm1, parm2, parm3) {
			for(var key in this.allWidgets) {
				var widget = this.allWidgets[key];
				if (dojo.isFunction(widget[functionName])) {
					widget[functionName](parm1, parm2, parm3);
				}				
			}
		}, 
		pollChildren: function(functionName, parm1, parm2, parm3) {
			for(var key in this.allWidgets) {
				var widget = this.allWidgets[key];
				if (dojo.isFunction(widget[functionName])) {
					if(widget[functionName](parm1, parm2, parm3)) {
						return true;
					};
				}				
			}
		},
		clearRetainedState: function() {
			this.notifyChildren("clearRetainedState");
		},

//		_handleNewCommentClick: function(e) {
//			e.preventDefault();
//			this._showNewCommentArea();
//		},
//
//		_showNewCommentArea: function() {
//			if(this.createCommentCmd.className == "AddCommentLinkDisabled") {
//				return;
//			}
//			this.newCommentArea.showNewComment();
//			this.createCommentCmd.className = "AddCommentLinkDisabled";
//			this.createCommentCmdBottom.className = "AddCommentLinkBottomDisabled";
//		},
//		_handleSaveCommentClick: function(e) {
//			e.preventDefault();
//			if(!this.parentController._isDirty) return;
//			this.parentController.save();
//		},
		//*********************************************************
		
		_handleDirtyingEvent: function(e) {
			this.parentController.conditionalMarkDirty(e);
		},
		markDirty: function() {
			this._isDirty = true;
		},		
		_handleSaveClick: function(e) {
			this.parentController._handleSaveClick(e);
		},
		handleSaveComplete: function() {
			this.notifyChildren("handleSaveComplete");
		}
	}
);

})();
